@@ -135,7 +135,6 @@ def download_rnx2_data(year, doy, site, out_dir=".", show_url=True):
         print(f"Failed to download rnx data for {year}-{doy}. Error: {e}")
 
 
-
 def download_rnx3_data(year, doy, site, out_dir=".", show_url=True):
     yr = year-2000
     # url=f"ftp://igs.gnsswhu.cn/pub/gps/data/daily/{year:04d}/{doy:03d}/{yr:02d}d/{site}{doy:03d}0.{yr:02d}d.Z"
@@ -397,7 +396,8 @@ def main():
         start_year, start_doy = args.start_date
         end_year, end_doy = args.end_date
         if args.data == "rnx2" or args.data == "rnx3":
-            site = args.site
+            sites = args.site.split(",")
+            # print(sites)
         start_date = datetime(start_year, 1, 1) + timedelta(days=start_doy - 1)
         end_date = datetime(end_year, 1, 1) + timedelta(days=end_doy - 1)
         last_month = 0
@@ -414,9 +414,11 @@ def main():
             elif args.data == 'clk':
                 download_clk_data(year, doy, out_dir=args.outdir, show_url=args.show_url)
             elif args.data=="rnx2":
-                download_rnx2_data(year, doy, site, out_dir=args.outdir, show_url=args.show_url)
+                for site in sites:
+                    download_rnx2_data(year, doy, site, out_dir=args.outdir, show_url=args.show_url)
             elif args.data=="rnx3":
-                download_rnx3_data(year, doy, site, out_dir=args.outdir, show_url=args.show_url)
+                for site in sites:
+                    download_rnx3_data(year, doy, site, out_dir=args.outdir, show_url=args.show_url)
             elif args.data=="igs.daily.snx":
                 download_igs_daily_snx(year, doy, out_dir=args.outdir, show_url=args.show_url)
             # elif args.data=="ivs.daily.snx":
