@@ -231,6 +231,8 @@ class TestUtils(unittest.TestCase):
         xr_gim = gnss_utils.loadobject(fn_gim)
         xr_obs = gnss_utils.loadobject(fn_obs)
         xr_obs = xr_obs.sel(site=["ahaq", "bjgb", "dxin", "ynws", "ynya", "zjjd"])
+
+        # xr_obs = xr_obs.sel(site=["ahaq"])
         
         pd_obs = gnss_utils.xr_obs2pd(xr_obs)
         pd_obs_slon = gnss_utils.pd_obs2slon(pd_obs)
@@ -241,9 +243,17 @@ class TestUtils(unittest.TestCase):
         # plt.close()  
         time_start = xr_obs.coords['time'][0].values
         time_end = xr_obs.coords['time'][-1].values
-        xr_gim = xr_gim.sel(time=slice(time_start, time_end))
-
-        pd_diff = gnss_utils.gim_vs_obs(xr_gim, xr_obs, plot=True)
+        xr_gim = xr_gim.sel(time=slice(time_start, time_end), lon=slice(-175, 180))
+        xr_gim_slon = gnss_utils.xr_gim2solar(xr_gim)
         
+        pd_diff = gnss_utils.valid_obs_vs_gim(pd_obs_slon, xr_gim_slon)
+        # pd_diff = gnss_utils.gim_vs_obs(xr_gim, xr_obs, plot=True)
+        gnss_utils.plot_gim_vs_obs(xr_gim_slon, pd_diff)
+        rms_groupby_site = pd_diff.groupby("site")["delta"].apply(lambda x: np.sqrt(np.nanmean(x**2)))
+        print(rms_groupby_site)
+        rms_groupby_time = pd_diff.groupby("ref_time_pre")["delta"].apply(lambda x: np.sqrt(np.nanmean(x**2)))
+        print(rms_groupby_time)
 
+        rms_groupby_site_and_time = pd_diff.groupby(["site", "ref_time_pre"])["delta"].apply(lambda x: np.sqrt(np.nanmean(x**2)))
+        print(rms_groupby_site_and_time)
         pass
